// Include Libraries
#include "Arduino.h"
#include "RFID.h"
#include "Servo.h"


// Pin Definitions
#define RFID_PIN_SDA	10
#define RFID_PIN_RST	2
#define SERVOMD_1_PIN_SIG	3
#define SERVOMD_2_PIN_SIG	4



// Global variables and defines
const int lockPosition   = 20;  //lock position
const int openPosition = 150; //open Position when event is detected

// object initialization
Servo servoMD_1;
Servo servoMD_2;
RFID rfid(RFID_PIN_SDA, RFID_PIN_RST);



// define vars for testing menu
const int timeout = 10000;       //define timeout of 10 sec
char menuOption = 0;
long time0;

// Setup the essentials for your circuit to work. It runs first every time your circuit is powered with electricity.
void setup()
{
  // Setup Serial which is useful for debugging
  // Use the Serial Monitor to view printed messages
  Serial.begin(9600);
  while (!Serial) ; // wait for serial port to connect. Needed for native USB
  Serial.println("start");

  //initialize RFID module
  rfid.init();
  servoMD_1.attach(SERVOMD_1_PIN_SIG);
  servoMD_2.attach(SERVOMD_2_PIN_SIG);
  servoMD_1.write(lockPosition);
  servoMD_2.write(lockPosition);
  delay(100);
  servoMD_1.detach();
  servoMD_2.detach();

}

// Main logic of your circuit. It defines the interaction between the components you selected. After setup, it runs over and over again, in an eternal loop.
void loop()
{

  // RFID - RC522 RF IC Card Sensor Module - Test Code
  //Read RFID tag if present
  String rfidtag = rfid.readTag();
  //print the tag to serial monitor if one was discovered
  rfid.printTag(rfidtag);

  if (rfidtag == "PUT YOUR RFID TAG HERE") {

    // if RFID tag detected the servos will move to open position for 2 secs and then move go back to lock position
    servoMD_1.attach(SERVOMD_1_PIN_SIG);         // 1. attach the servo to correct pin to control it.
    servoMD_2.attach(SERVOMD_2_PIN_SIG);         // 1. attach the servo to correct pin to control it.
    servoMD_1.write(openPosition);  // 2. turns servo to target position. Modify target position by modifying the 'ServoTargetPosition' definition above.
    servoMD_2.write(openPosition);  // 2. turns servo to target position. Modify target position by modifying the 'ServoTargetPosition' definition above.
    delay(2000); //wait 2 secs and lock the servos
    servoMD_1.write(lockPosition);  // 2. turns servo to target position. Modify target position by modifying the 'ServoTargetPosition' definition above.
    servoMD_2.write(lockPosition);  // 2. turns servo to target position. Modify target position by modifying the 'ServoTargetPosition' definition above.
    servoMD_1.detach();                    // 6. release the servo to conserve power. When detached the servo will NOT hold it's position under stress.
    servoMD_2.detach();                    // 6. release the servo to conserve power. When detached the servo will NOT hold it's position under stress.

  }
}


/*******************************************************

     Circuito.io is an automatic generator of schematics and code for off
     the shelf hardware combinations.

     Copyright (C) 2016 Roboplan Technologies Ltd.

     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.

     In addition, and without limitation, to the disclaimers of warranties
     stated above and in the GNU General Public License version 3 (or any
     later version), Roboplan Technologies Ltd. ("Roboplan") offers this
     program subject to the following warranty disclaimers and by using
     this program you acknowledge and agree to the following:
     THIS PROGRAM IS PROVIDED ON AN "AS IS" AND "AS AVAILABLE" BASIS, AND
     WITHOUT WARRANTIES OF ANY KIND EITHER EXPRESS OR IMPLIED.  ROBOPLAN
     HEREBY DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT
     NOT LIMITED TO IMPLIED WARRANTIES OF MERCHANTABILITY, TITLE, FITNESS
     FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT, AND THOSE ARISING BY
     STATUTE OR FROM A COURSE OF DEALING OR USAGE OF TRADE.
     YOUR RELIANCE ON, OR USE OF THIS PROGRAM IS AT YOUR SOLE RISK.
     ROBOPLAN DOES NOT GUARANTEE THAT THE PROGRAM WILL BE FREE OF, OR NOT
     SUSCEPTIBLE TO, BUGS, SECURITY BREACHES, OR VIRUSES. ROBOPLAN DOES
     NOT WARRANT THAT YOUR USE OF THE PROGRAM, INCLUDING PURSUANT TO
     SCHEMATICS, INSTRUCTIONS OR RECOMMENDATIONS OF ROBOPLAN, WILL BE SAFE
     FOR PERSONAL USE OR FOR PRODUCTION OR COMMERCIAL USE, WILL NOT
     VIOLATE ANY THIRD PARTY RIGHTS, WILL PROVIDE THE INTENDED OR DESIRED
     RESULTS, OR OPERATE AS YOU INTENDED OR AS MAY BE INDICATED BY ROBOPLAN.
     YOU HEREBY WAIVE, AGREE NOT TO ASSERT AGAINST, AND RELEASE ROBOPLAN,
     ITS LICENSORS AND AFFILIATES FROM, ANY CLAIMS IN CONNECTION WITH ANY OF
     THE ABOVE.
********************************************************/
